@@ -22,7 +22,15 @@ extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,INT (./rb-marshal.list:3) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:3) */
+extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
+
+/* VOID:STRING,INT (./rb-marshal.list:4) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -30,7 +38,7 @@ extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:STRING,STRING (./rb-marshal.list:4) */
+/* VOID:STRING,STRING (./rb-marshal.list:5) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -38,7 +46,7 @@ extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:5) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:6) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -46,7 +54,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* BOOLEAN:POINTER (./rb-marshal.list:6) */
+/* BOOLEAN:POINTER (./rb-marshal.list:7) */
 extern void rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -54,9 +62,33 @@ extern void rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:DOUBLE (./rb-marshal.list:7) */
+/* VOID:DOUBLE (./rb-marshal.list:8) */
 #define rb_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
 
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:9) */
+extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
+                                                  GValue       *return_value,
+                                                  guint         n_param_values,
+                                                  const GValue *param_values,
+                                                  gpointer      invocation_hint,
+                                                  gpointer      marshal_data);
+
+/* VOID:INT,POINTER (./rb-marshal.list:10) */
+extern void rb_marshal_VOID__INT_POINTER (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data);
+
+/* VOID:POINTER,INT,POINTER,POINTER (./rb-marshal.list:11) */
+extern void rb_marshal_VOID__POINTER_INT_POINTER_POINTER (GClosure     *closure,
+                                                          GValue       *return_value,
+                                                          guint         n_param_values,
+                                                          const GValue *param_values,
+                                                          gpointer      invocation_hint,
+                                                          gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __rb_marshal_MARSHAL_H__ */
