@@ -1,6 +1,8 @@
 /*
+ *  arch-tag: Header with definitions of various GConf keys
+ *
  *  Copyright (C) 2002 Jorn Baayen
- *  Copyright (C) 2003 Colin Walters <cwalters@gnome.org>
+ *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-preferences.h,v 1.5 2003/08/19 00:10:44 cwalters Exp $
  */
 
 #ifndef __RB_PREFERENCES_H
@@ -27,13 +28,16 @@ G_BEGIN_DECLS
 #define CONF_PREFIX "/apps/rhythmbox"
 
 #define CONF_FIRST_TIME CONF_PREFIX "/first_time_flag"
+#define CONF_AUTO_RATE  CONF_PREFIX "/auto_rate"
 #define CONF_UI_DIR               CONF_PREFIX "/ui"
 #define CONF_UI_STATUSBAR_HIDDEN  CONF_PREFIX "/ui/statusbar_hidden"
 #define CONF_UI_SOURCELIST_HIDDEN CONF_PREFIX "/ui/sourcelist_hidden"
-#define CONF_UI_COLUMNS_SETUP CONF_PREFIX "/ui/columns_setup"
+#define CONF_UI_SMALL_DISPLAY     CONF_PREFIX "/ui/small_display"
+#define CONF_UI_OBSOLETE_COLUMNS_SETUP CONF_PREFIX "/ui/columns_setup"
+#define CONF_UI_COLUMNS_SETUP CONF_PREFIX "/ui/rhythmdb_columns_setup"
+#define CONF_UI_TIME_DISPLAY CONF_PREFIX "/ui/time_display"
 #define CONF_UI_BROWSER_VIEWS CONF_PREFIX "/ui/browser_views"
-#define CONF_STATE_SHUFFLE	CONF_PREFIX "/state/shuffle"
-#define CONF_STATE_REPEAT	CONF_PREFIX "/state/repeat"
+#define CONF_STATE_PLAY_ORDER	CONF_PREFIX "/state/play_order"
 #define CONF_STATE_VOLUME	CONF_PREFIX "/state/volume"
 
 G_END_DECLS
