@@ -23,6 +23,8 @@
 #define __RB_SHELL_H
 
 #include "Rhythmbox.h"
+#include "rb-source.h"
+#include "rhythmdb.h"
 
 #include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-object.h>
@@ -58,11 +60,17 @@ typedef struct
 GType		rb_shell_get_type	(void);
 
 RBShell *	rb_shell_new		(int argc, char **argv,
-					 gboolean no_registration, gboolean dry_run,
+					 gboolean no_registration,
+					 gboolean no_update,
+					 gboolean dry_run,
 					 char *rhythmdb);
 
 void		rb_shell_construct	(RBShell *shell);
 
+void            rb_shell_register_entry_type_for_source (RBShell *shell,
+							 RBSource *source,
+							 RhythmDBEntryType type);
+
 /* utilities */
 
 char *		rb_shell_corba_exception_to_string (CORBA_Environment *ev);
