@@ -31,14 +31,14 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include <stdlib.h>
-
-#include "monkey-media-audio-quality.h"
+#include <math.h>
 
 #include "rb-tree-dnd.h"
 #include "rb-tree-view-column.h"
 #include "rb-entry-view.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
+#include "rb-util.h"
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-cell-renderer-pixbuf.h"
@@ -104,16 +104,16 @@ static void rb_entry_view_sort_key_changed_cb (GConfClient* client,
 /* 						  gpointer user_data); */
 static void rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 				   const char *path,
-				   int rating,
+				   double rating,
 				   RBEntryView *view);
 static gboolean rb_entry_view_button_press_cb (GtkTreeView *treeview,
 					      GdkEventButton *event,
 					      RBEntryView *view);
-static gboolean rb_entry_view_entry_is_visible (RBEntryView *view, RhythmDBEntry *entry,
-						GtkTreeIter *iter);
+static void rb_entry_view_entry_is_visible (RBEntryView *view, RhythmDBEntry *entry,
+					    gboolean *realized, gboolean *visible,
+					    GtkTreeIter *iter);
 static void rb_entry_view_scroll_to_iter (RBEntryView *view,
 					  GtkTreeIter *iter);
-static gboolean idle_poll_model (RBEntryView *view);
 
 struct RBEntryViewReverseSortingData
 {
@@ -150,6 +150,7 @@ struct RBEntryViewPrivate
 	GtkTreeViewColumn *sorting_column;
 	gint sorting_order;
 	struct RBEntryViewReverseSortingData *reverse_sorting_data;
+	gboolean resorting;
 
 	gboolean have_selection;
 
@@ -169,18 +170,13 @@ struct RBEntryViewPrivate
 	GHashTable *column_sort_data_map;
 
 	gboolean idle;
-
-	guint model_poll_id;
-
-#ifdef USE_GTK_TREE_VIEW_WORKAROUND	
-	guint freeze_count;
-#endif
 };
 
 enum
 {
 	ENTRY_ADDED,
 	ENTRY_DELETED,
+	ENTRIES_REPLACED,
 	ENTRY_SELECTED,
 	ENTRY_ACTIVATED,
 	CHANGED,
@@ -309,6 +305,15 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
+	rb_entry_view_signals[ENTRIES_REPLACED] =
+		g_signal_new ("entries-replaced",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBEntryViewClass, entries_replaced),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	rb_entry_view_signals[ENTRY_ACTIVATED] =
 		g_signal_new ("entry-activated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -380,21 +385,13 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 static void
 rb_entry_view_init (RBEntryView *view)
 {
-	GtkWidget *dummy;
-	
 	view->priv = g_new0 (RBEntryViewPrivate, 1);
 
-	dummy = gtk_tree_view_new ();
-	view->priv->playing_pixbuf = gtk_widget_render_icon (dummy,
-							     RB_STOCK_PLAYING,
-							     GTK_ICON_SIZE_MENU,
-							     NULL);
+	view->priv->playing_pixbuf = rb_pixbuf_new_from_stock (RB_STOCK_PLAYING,
+							       GTK_ICON_SIZE_MENU);
 
-	view->priv->paused_pixbuf = gtk_widget_render_icon (dummy,
-							    RB_STOCK_PAUSED,
-							    GTK_ICON_SIZE_MENU,
-							    NULL);
-	gtk_widget_destroy (dummy);
+	view->priv->paused_pixbuf = rb_pixbuf_new_from_stock (RB_STOCK_PAUSED,
+							      GTK_ICON_SIZE_MENU);
 
 	view->priv->propid_column_map = g_hash_table_new (NULL, NULL);
 	view->priv->column_sort_data_map = g_hash_table_new_full (NULL, NULL, NULL, g_free);
@@ -416,8 +413,6 @@ rb_entry_view_finalize (GObject *object)
 	if (view->priv->change_sig_queued)
 		g_source_remove (view->priv->change_sig_id);
 
-	g_source_remove (view->priv->model_poll_id);
-
 	if (view->priv->gconf_notification_id > 0)
 		eel_gconf_notification_remove (view->priv->gconf_notification_id);
 	if (view->priv->sorting_gconf_notification_id > 0)
@@ -432,88 +427,14 @@ rb_entry_view_finalize (GObject *object)
 	g_object_unref (G_OBJECT (view->priv->playing_pixbuf));
 	g_object_unref (G_OBJECT (view->priv->paused_pixbuf));
 
+	g_free (view->priv->sorting_key);
+
 	g_free (view->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
-/* FIXME!
- * This is a gross hack to work around a GTK+ bug.  See
- * http://bugzilla.gnome.org/show_bug.cgi?id=119797 for more
- * information.
- */
-void
-rb_entry_view_thaw (RBEntryView *view)
-{
-/* RHYTHMDB FIXME */
-#if 0
-#ifdef USE_GTK_TREE_VIEW_WORKAROUND
-	GList *columns;
-	int i;
-
-	g_return_if_fail (view->priv->freeze_count > 0);
-
-	view->priv->freeze_count--;
-	if (view->priv->freeze_count > 0)
-		return;
-
-	rb_debug ("thawing entry view");
-
-	columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (view->priv->treeview));
-
-	for (i = 0; i < RB_TREE_MODEL_NODE_NUM_COLUMNS && columns != NULL; i++, columns = g_list_next (columns))
-		switch (i)
-		{
-		case RB_TREE_MODEL_NODE_COL_PLAYING:
-			break;
-		case RB_TREE_MODEL_NODE_COL_RATING:
-			break;
-		default:
-			gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (columns->data),
-							 GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-
-			break;
-		}
-	g_list_free (columns);
-#endif
-#endif
-}
-
-void
-rb_entry_view_freeze (RBEntryView *view)
-{
-/* RHYTHMDB FIXME */
-#if 0
-#ifdef USE_GTK_TREE_VIEW_WORKAROUND
-	GList *columns;
-	int i;
-
-	view->priv->freeze_count++;
-
-	rb_debug ("freezing entry view");
-	
-	columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (view->priv->treeview));
-	
-	for (i = 0; i < RB_TREE_MODEL_NODE_NUM_COLUMNS && columns != NULL; i++, columns = g_list_next (columns))
-		switch (i)
-		{
-		case RB_TREE_MODEL_NODE_COL_PLAYING:
-			break;
-		case RB_TREE_MODEL_NODE_COL_RATING:
-			break;
-		default:
-			gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (columns->data),
-							 GTK_TREE_VIEW_COLUMN_FIXED);
-
-			break;
-		}
-
-	g_list_free (columns);
-#endif
-#endif
-}
-
 static void
 rb_entry_view_set_property (GObject *object,
 			   guint prop_id,
@@ -528,6 +449,7 @@ rb_entry_view_set_property (GObject *object,
 		view->priv->db = g_value_get_object (value);
 		break;
 	case PROP_SORTING_KEY:
+		g_free (view->priv->sorting_key);
 		view->priv->sorting_key = g_value_dup_string (value);
 		break;
 	case PROP_MODEL:
@@ -538,6 +460,15 @@ rb_entry_view_set_property (GObject *object,
 		if (view->priv->model) {
 			rhythmdb_query_model_cancel (view->priv->model);
 			rhythmdb_query_model_set_connected (RHYTHMDB_QUERY_MODEL (view->priv->model), FALSE);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_inserted_cb),
+							      view);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_deleted_cb),
+							      view);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_changed_cb),
+							      view);
 		}
 		new_model = g_value_get_object (value);
 
@@ -585,6 +516,15 @@ rb_entry_view_set_property (GObject *object,
 					 GTK_TREE_MODEL (new_model));
 		view->priv->model = new_model;
 		view->priv->have_selection = FALSE;
+
+		if (view->priv->resorting) {
+			/* When the sort order changes, the model is replaced
+			 * but the set of entries doesn't change. */
+			view->priv->resorting = FALSE;
+		} else {
+			g_signal_emit (G_OBJECT (view), rb_entry_view_signals[ENTRIES_REPLACED], 0);
+		}
+
 		queue_changed_sig (view);
 
 		break;
@@ -594,7 +534,7 @@ rb_entry_view_set_property (GObject *object,
 		GtkTreeIter iter;
 		GtkTreePath *path;
 		RhythmDBEntry *entry;
-		rb_entry_view_freeze (view);
+		gboolean realized, visible;
 
 		entry = g_value_get_pointer (value);
 		
@@ -626,12 +566,13 @@ rb_entry_view_set_property (GObject *object,
 			}
 		}
 
-		rb_entry_view_thaw (view);
-
 		if (view->priv->playing_entry
-		    && view->priv->playing_entry_in_view
-		    && !rb_entry_view_entry_is_visible (view, view->priv->playing_entry, &iter))
-			rb_entry_view_scroll_to_iter (view, &iter);
+		    && view->priv->playing_entry_in_view) {
+		    rb_entry_view_entry_is_visible (view, view->priv->playing_entry,
+						    &realized, &visible, &iter);
+		    if (realized && !visible)
+			    rb_entry_view_scroll_to_iter (view, &iter);
+		}
 	}
 	break;
 	case PROP_IS_DRAG_SOURCE:
@@ -731,7 +672,7 @@ entry_from_tree_path (RBEntryView *view, GtkTreePath *path)
 	GtkTreeIter entry_iter;
 	RhythmDBEntry *entry;
 
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->model), &entry_iter, path);
+	g_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->model), &entry_iter, path));
 	gtk_tree_model_get (GTK_TREE_MODEL (view->priv->model), &entry_iter, 0,
 			    &entry, -1);
 	return entry;
@@ -746,13 +687,6 @@ entry_from_tree_iter (RBEntryView *view, GtkTreeIter *iter)
 	return entry;
 }
 
-RhythmDBQueryModel *
-rb_entry_view_get_model (RBEntryView *view)
-{
-	g_return_val_if_fail (RB_IS_ENTRY_VIEW (view), NULL);
-	return view->priv->model;
-}
-
 static gint
 reverse_sorting_func (gpointer a, gpointer b, struct RBEntryViewReverseSortingData *data)
 {
@@ -774,6 +708,7 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	const char *b_str = NULL;
 	gint ret;
 
+	/* Sort by album name */
 	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
 	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
 
@@ -781,12 +716,20 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	if (ret != 0)
 		return ret;
 
+	/* Then by disc number, */
+	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_DISC_NUMBER);
+	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_DISC_NUMBER);
+	if (a_int != b_int)
+		return (a_int < b_int ? -1 : 1);
+
+	/* by track number */
 	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
 	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
 
 	if (a_int != b_int)
 		return (a_int < b_int ? -1 : 1);
 
+	/* And finally by title */
 	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_TITLE_SORT_KEY);
 	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_TITLE_SORT_KEY);
 
@@ -833,28 +776,7 @@ static gint
 rb_entry_view_track_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
-	gint a_int, b_int;
-	const char *a_str = NULL;
-	const char *b_str = NULL;	
-	gint ret;
-
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
-	if (ret != 0)
-		goto out;
-
-	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
-	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
-
-	if (a_int != b_int) {
-		ret = (a_int < b_int ? -1 : 1);
-		goto out;
-	}
-
-out:
-	return ret;
+	return rb_entry_view_album_sort_func (a, b, view);
 }
 
 
@@ -873,6 +795,21 @@ rb_entry_view_int_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	return ret;
 }
 
+static gint
+rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+				       struct RBEntryViewCellDataFuncData *data)
+{
+	gdouble a_val, b_val;
+	gint ret;
+
+	a_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, a, data->propid));
+	b_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, b, data->propid));
+
+	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
+
+	return ret;
+}
+
 static gint
 rb_entry_view_long_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			      struct RBEntryViewCellDataFuncData *data)
@@ -930,13 +867,13 @@ rb_entry_view_rating_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 				     RBEntryView *view)
 {
 	RhythmDBEntry *entry;
-	guint rating;
+	gdouble rating;
 
 	entry = entry_from_tree_iter (view, iter);
 
 	rhythmdb_read_lock (view->priv->db);
 
-	rating = rhythmdb_entry_get_int (view->priv->db, entry, RHYTHMDB_PROP_RATING);
+	rating = rhythmdb_entry_get_double (view->priv->db, entry, RHYTHMDB_PROP_RATING);
 
 	rhythmdb_read_unlock (view->priv->db);
 
@@ -1016,7 +953,10 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 	minutes = duration / 60;
 	seconds = duration % 60;
 
-	str = g_strdup_printf (_("%d:%02d"), minutes, seconds);
+	if (minutes == 0 && seconds == 0)
+		str = g_strdup (_("Unknown"));
+	else
+		str = g_strdup_printf (_("%d:%02d"), minutes, seconds);
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 
@@ -1029,24 +969,31 @@ rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 				      struct RBEntryViewCellDataFuncData *data)
 {
 	RhythmDBEntry *entry;
-	char *str;
-	MonkeyMediaAudioQuality quality;
+	guint bitrate;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
 	rhythmdb_read_lock (data->view->priv->db);
 
-	quality = rhythmdb_entry_get_int (data->view->priv->db, entry,
+	bitrate = rhythmdb_entry_get_int (data->view->priv->db, entry,
 					  data->propid);
 
 	rhythmdb_read_unlock (data->view->priv->db);
 
-	if (quality > 0) {
-		str = monkey_media_audio_quality_to_string (quality);
-		g_object_set (G_OBJECT (renderer), "text", str, NULL);
-		g_free (str);
-	} else {
+	if (bitrate <= 0) {
 		g_object_set (G_OBJECT (renderer), "text", _("Unknown"), NULL);
+	} else if (bitrate <= 80) {
+		g_object_set (G_OBJECT (renderer), "text", _("Very Low"), NULL);
+	} else if (bitrate <= 112) {
+		g_object_set (G_OBJECT (renderer), "text", _("Low"), NULL);
+	} else if (bitrate <= 160) {
+		g_object_set (G_OBJECT (renderer), "text", _("Regular"), NULL);
+	} else if (bitrate <= 224) {
+		g_object_set (G_OBJECT (renderer), "text", _("High"), NULL);
+	} else if (bitrate <= 1410) {
+		g_object_set (G_OBJECT (renderer), "text", _("Very High"), NULL);
+	} else {
+		g_object_set (G_OBJECT (renderer), "text", _("Perfect"), NULL);
 	}
 }
 
@@ -1134,6 +1081,14 @@ rb_entry_view_get_sorting_type (RBEntryView *view)
 	return eel_gconf_get_string (view->priv->sorting_key);
 }
 
+void
+rb_entry_view_set_resorting (RBEntryView *view)
+{
+	if (view->priv->resorting)
+		g_warning ("Sort order changed while resorting");
+	view->priv->resorting = TRUE;
+}
+
 static void
 rb_entry_view_column_clicked_cb (GtkTreeViewColumn *column, RBEntryView *view)
 {
@@ -1198,7 +1153,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		sort_data = g_new0 (struct RBEntryViewCellDataFuncData, 1);
 		sort_data->view = view;
 		sort_data->propid = propid;
-		sort_func = (GCompareDataFunc) rb_entry_view_int_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_double_ceiling_sort_func;
 
 		renderer = rb_cell_renderer_rating_new ();
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -1291,7 +1246,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		key = "Time";
 		break;
 	case RB_ENTRY_VIEW_COL_QUALITY:
-		propid = RHYTHMDB_PROP_QUALITY;
+		propid = RHYTHMDB_PROP_BITRATE;
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_quality_cell_data_func;
 		sort_data->propid = cell_data->propid;
@@ -1362,6 +1317,7 @@ rb_entry_view_append_column_custom (RBEntryView *view,
 				g_strdup (key), g_free);
 
 	rb_debug ("appending column: %p (%s)", column, title);
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
 
 	sortdata = g_new (struct RBEntryViewColumnSortData, 1);
@@ -1392,7 +1348,7 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 	RBEntryView *view;
 	RBEntryViewClass *klass;
 	GObjectClass *parent_class;
-	klass = RB_ENTRY_VIEW_CLASS (g_type_class_peek (type));
+	klass = RB_ENTRY_VIEW_CLASS (g_type_class_peek (RB_TYPE_ENTRY_VIEW));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	view = RB_ENTRY_VIEW (parent_class->constructor (type, n_construct_properties,
@@ -1479,8 +1435,6 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 		g_object_unref (G_OBJECT (query_model));
 	}
 		
-	view->priv->model_poll_id = g_idle_add ((GSourceFunc) idle_poll_model, view);
-
 	return G_OBJECT (view);
 }
 
@@ -1500,29 +1454,32 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 static void
 rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 		       const char *path_string,
-		       int rating,
+		       double rating,
 		       RBEntryView *view)
 {
 	GtkTreePath *path;
 	RhythmDBEntry *entry;
 	GValue value = { 0, };
 
-	g_return_if_fail (rating >= 1 && rating <= 5 );
+	g_return_if_fail (rating >= 0 && rating <= 5 );
 	g_return_if_fail (path_string != NULL);
 
 	path = gtk_tree_path_new_from_string (path_string);
 	entry = entry_from_tree_path (view, path);
 	gtk_tree_path_free (path);
 
-	rb_entry_view_freeze (view);
-
-	g_value_init (&value, G_TYPE_INT);
-	g_value_set_int (&value, rating);
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, rating);
 	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_RATING,
 				  &value);
 	g_value_unset (&value);
 
-	rb_entry_view_thaw (view);
+	/* since the user changed the rating, stop auto-rating */
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, FALSE);
+	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_AUTO_RATE,
+				  &value);
+	g_value_unset (&value);
 }
 
 void
@@ -1578,6 +1535,7 @@ rb_entry_view_get_previous_from_entry (RBEntryView *view, RhythmDBEntry *entry)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
+	RhythmDBEntry *ret;
 
 	g_return_val_if_fail (entry != NULL, NULL);
 
@@ -1593,7 +1551,9 @@ rb_entry_view_get_previous_from_entry (RBEntryView *view, RhythmDBEntry *entry)
 	}
 
 	g_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->model), &iter, path));
-	return entry_from_tree_iter (view, &iter);
+	gtk_tree_path_free (path);
+	ret = entry_from_tree_iter (view, &iter);
+	return ret;
 }
 
 
@@ -1771,6 +1731,10 @@ rb_entry_view_row_inserted_cb (GtkTreeModel *model,
 			       GtkTreeIter *iter,
 			       RBEntryView *view)
 {
+	RhythmDBEntry *entry = entry_from_tree_path (view, path);
+
+	rb_debug ("row added");
+	g_signal_emit (G_OBJECT (view), rb_entry_view_signals[ENTRY_ADDED], 0, entry);
 	queue_changed_sig (view);
 }
 
@@ -1884,11 +1848,14 @@ rb_entry_view_get_entry_visible (RBEntryView *view,
 				 RhythmDBEntry *entry)
 {
 	GtkTreeIter unused;
+	gboolean realized, visible;
 
 	if (view->priv->playing_model != view->priv->model)
 		return FALSE;
 
-	return rb_entry_view_entry_is_visible (view, entry, &unused);
+	rb_entry_view_entry_is_visible (view, entry, &realized, &visible,
+					&unused);
+	return realized && visible;
 }
 
 gboolean
@@ -1901,22 +1868,29 @@ rb_entry_view_get_entry_contained (RBEntryView *view,
 						   entry, &unused);
 }
 
-static gboolean
+static void
 rb_entry_view_entry_is_visible (RBEntryView *view,
 				RhythmDBEntry *entry,
+				gboolean *realized,
+				gboolean *visible,
 				GtkTreeIter *iter)
 {
 	GtkTreePath *path;
 	GdkRectangle rect;
 
-	g_return_val_if_fail (entry != NULL, FALSE);
+	*realized = FALSE;
+	*visible = FALSE;
+
+	g_return_if_fail (entry != NULL);
 
 	if (!GTK_WIDGET_REALIZED (view))
-		return FALSE;
+		return;
+
+	*realized = TRUE;
 
 	if (!rhythmdb_query_model_entry_to_iter (view->priv->model,
 						 entry, iter))
-		return FALSE;
+		return;
 
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model), iter);
 	gtk_tree_view_get_cell_area (GTK_TREE_VIEW (view->priv->treeview),
@@ -1926,7 +1900,7 @@ rb_entry_view_entry_is_visible (RBEntryView *view,
 
 	gtk_tree_path_free (path);
 
-	return rect.y != 0 && rect.height != 0;
+	*visible = (rect.y != 0 && rect.height != 0);
 }
 
 static gboolean
@@ -2035,8 +2009,8 @@ rb_entry_view_sync_columns_visible (RBEntryView *view)
 		for (i = 0; items[i] != NULL && *(items[i]); i++) {
 			int value = propid_from_name (items[i]);
 
-			g_assert ((value >= 0) && (value < RHYTHMDB_NUM_PROPERTIES));
-			visible_properties = g_list_append (visible_properties, GINT_TO_POINTER (value));
+			if ((value >= 0) && (value < RHYTHMDB_NUM_PROPERTIES))
+				visible_properties = g_list_append (visible_properties, GINT_TO_POINTER (value));
 		}
 		g_strfreev (items);
 	}
@@ -2058,52 +2032,17 @@ rb_entry_view_set_playing (RBEntryView *view,
 {
 	g_return_if_fail (RB_IS_ENTRY_VIEW (view));
 
-	rb_entry_view_freeze (view);
-
 	view->priv->playing = TRUE;
-
-	rb_entry_view_thaw (view);
 }
 
 gboolean
 rb_entry_view_poll_model (RBEntryView *view)
 {
 	GTimeVal timeout;
-	gboolean did_sync;
 
 	g_get_current_time (&timeout);
 	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
 
-	did_sync = rhythmdb_query_model_poll (view->priv->model, &timeout);
-	if (did_sync) {
-		g_source_remove (view->priv->model_poll_id);
-		view->priv->model_poll_id =
-			g_idle_add ((GSourceFunc) idle_poll_model, view);
-	}
-	return did_sync;
+	return rhythmdb_query_model_poll (view->priv->model, &timeout);
 }
 
-static gboolean
-idle_poll_model (RBEntryView *view)
-{
-	gboolean did_sync;
-	GTimeVal timeout;
-
-	g_get_current_time (&timeout);
-	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
-
-	GDK_THREADS_ENTER ();
-
-	did_sync = rhythmdb_query_model_poll (view->priv->model, &timeout);
-
-	if (did_sync)
-		view->priv->model_poll_id =
-			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) idle_poll_model, view, NULL);
-	else
-		view->priv->model_poll_id =
-			g_timeout_add (300, (GSourceFunc) idle_poll_model, view);
-
-	GDK_THREADS_LEAVE ();
-
-	return FALSE;
-}
